unit Tautils;

interface
Uses Classes;

Var
  Interrupt_Flag                                   :Boolean;

function StripSpacesInFront(InString: String):String;
function StripAllSpaces(InString: String):String;
function IsDirEmpty(TreeBranch: String):Boolean;
function GetParentDir(TreeBranch: String):String;
procedure DeleteTreeBranch(TreeBranch :String);
Function ItemIsDirectory(Dir, CurPath :String):Boolean;
Function GetFileName(InFileName :String):String;
Function PostFixDir(CurDir :String): String;
procedure CopyMoveTreeBranch(DirOp :Char; FromDir, ToDir :String; IncSubDir, IncFiles :Boolean);
Function ExtractCurDirName(TreeBranch :String):String;
Procedure CreateDirList(Var StrList, NameList :TStringList; Const DirPath :String; Const FileAttr :Word);
Procedure CheckforInterrupt(Key :Char; ObjectToInterrupt :String);
function WinExecAndWait(Const CmdStr: String; Const ShowMode: Word):Word;
function TADirectoryExists(Name: string): Boolean;




implementation
Uses sysutils, Dialogs, FileCtrl, WINPROCS, WINTypes, FileUtil, main, FindFile;


{ This Function is used to strip blanks away from the begining of a Pascal
    String
}
function StripSpacesInFront(InString: String):String;
Var
   TempString           :String;
   StrLength, Index1    :Integer;
   EndofFirstSpaces     :Boolean;
Begin
    StrLength := Length(InString);
    EndofFirstSpaces := False;
    Index1 := 1;
    TempString := '';
    While Index1 <= StrLength do Begin
       If  ((InString[Index1] <> ' ') or (EndofFirstSpaces)) then Begin
          TempString := TempString + InString[Index1];
          EndofFirstSpaces := True;
       End;
       Inc(Index1);
    End;
    Result := TempString;
End;


{ This Function is used to strip blanks away from both the beginning of a Pascal
    String and the end
}
function StripAllSpaces(InString: String):String;
Var
   TempString           :String;
   StrLength, Index1    :Integer;
Begin
    StrLength := Length(InString);
    Index1 := 1;
    TempString := '';
    While Index1 <= StrLength do Begin
       If  (InString[Index1] <> ' ') then Begin
          TempString := TempString + InString[Index1];
       End;
       Inc(Index1);
    End;
    Result := TempString;
End;


{ Checks to See of a Tree Branch is Empty }
function IsDirEmpty(TreeBranch: String):Boolean;
var
  SearchRec                     :TSearchRec;

begin
   If TreeBranch[Length(TreeBranch)] <> '\' Then
       TreeBranch := TreeBranch + '\';

   If FindFirst(TreeBranch + '*.*',faAnyFile, SearchRec) = 0 then begin
      Result := False;
   End Else Begin
      Result := True;
   End;
end;

{ Gets the Parent Directory of a Tree Branch }
function GetParentDir(TreeBranch: String):String;
var
  i      :Integer;
begin
   If (TreeBranch[Length(TreeBranch)] = '\') and (Length(TreeBranch) > 3) Then Begin
       TreeBranch := Copy(TreeBranch, 1, Length(TreeBranch) -1);
   End;

   for i := Length(TreeBranch) Downto 1 do Begin
      If TreeBranch[i] = '\' then
         Break;
   End;
   Result := Copy(TreeBranch, 1, i - 1);
end;




procedure DeleteTreeBranch(TreeBranch :String);
var
  FSearchRec, DSearchRec               :TSearchRec;
  FindResult                           :Integer;
  FileAttr                             :Byte;

Begin

  TreeBranch := PostFixDir(TreeBranch);

  FileAttr := 0;
  FileAttr := FileAttr + faHidden;
  FileAttr := FileAttr + faSysFile;
  FileAttr := FileAttr + faReadOnly;
{  FileAttr := FileAttr + faVolumeId;} { Do not want Volume ID - OS/2 Problems }
                                       { Besides - you can't delete a vol id   }

  FindResult := FindFirst(TreeBranch + '*.*' ,FileAttr, FSearchRec);

  Try
   { Chdir(TreeBranch);  Chg to that dir so that the DELETEFILE cmd will work }
    While (FindResult=0) Do Begin
      Try
        IF Not DeleteFile(TreeBranch + FSearchRec.Name) then Begin
          MessageDlg('File: ' + TreeBranch + FSearchRec.Name + ' not deleted!', mtInformation, [mbOK], 0);
          { Break;}
        End;
      Finally
        FindResult := FindNext(FSearchRec);
      End;
    End;

    FindResult := FindFirst(TreeBranch + '*.*', faDirectory, DSearchRec);

    While (FindResult=0) Do Begin
      If ((DSearchRec.Attr and faDirectory) = faDirectory)  and (pos('.', DSearchRec.Name) = 0) then begin
        DeleteTreeBranch(TreeBranch + DSearchRec.Name);
      End;
      FindResult := FindNext(DSearchRec);
    End;
    {$I-}
    RmDir(Copy(TreeBranch, 1, Length(TreeBranch) - 1));
    If IOResult <> 0 then Begin
      MessageDlg('Error Deleting Directory: ' + TreeBranch, mtInformation, [mbOK], 0);
    End;
  Finally
    FindClose(FSearchRec);
    FindClose(DSearchRec);
  End;
End;


{ This Function is used to determine if a item in a file list box is a directory
    or if it is just a normal file
}
function ItemIsDirectory(Dir, CurPath: String):Boolean;
Var
   DirName                    :String;
Begin
    DirName := Copy(Dir, 2, Length(Dir) - 2);
    If DirectoryExists(CurPath + '\' + DirName) then Begin
       Result := True;
    End Else Begin
    { Now check to see if Dir is '[..]' - if so - then make it yes. }
      If DirName = '..' then begin
        Result := True;
      End else begin
        Result := False;
      End;
    End;
End;

Function GetFileName(InFileName :String):String;
var
   Position    :Integer;
Begin
   Position := Pos(' ',InFileName);
   If Position > 0 then Begin
      Result := Copy(InFileName,1,Position - 1);
   end else begin
      Result := ' ';
   End;
End;



{ This function will append a '\' onto the end of a directory string
    if it is not already there
}
Function PostFixDir(CurDir :String): String;
Begin
   If CurDir[Length(CurDir)] <> '\' Then Begin
      Result := CurDir + '\';
   End Else Begin
      Result := CurDir;
   End;
End;

{ This procedure will copy a directory structure from one tree branch to another
       tree branch.
}
procedure CopyMoveTreeBranch(DirOp :Char; FromDir, ToDir :String; IncSubDir, IncFiles :Boolean);
Var
  NewDir, NewToDir, NewFromDir         :String;
  FSearchRec, DSearchRec               :TSearchRec;
  FindResult                           :Integer;
  FileAttr                             :Byte;

Begin
  NewDir := PostFixDir(ToDir) + ExtractCurDirName(FromDir);
  If Not DirectoryExists(NewDir) then begin
     MKDIR(NewDir);
     If IOResult <> 0 then begin
        MessageDlg('Directory Copy/Move Aborted - Could not create Directory ' +
                    NewDir, mtWarning, [mbOK],0);
     End;
  End;

{ Find All Files }
  FileAttr := 0;
  FileAttr := FileAttr + faHidden;
  FileAttr := FileAttr + faSysFile;
  FileAttr := FileAttr + faReadOnly;
{  FileAttr := FileAttr + faVolumeId;}

  FindResult := FindFirst(PostFixDir(FromDir) + '*.*' ,FileAttr, FSearchRec);

  Try
    { Only Copy the files if the INCFILES Flag is set }
    If IncFiles then Begin
      While (FindResult=0) Do Begin
      { Now Copy or Move the Found File to the New Directory }
        IF DirOp = 'C' then Begin
           CopyFile(PostFixDir(FromDir) + FSearchRec.Name, PostFixDir(NewDir) + FSearchRec.Name);
        End Else Begin
           MoveFile(PostFixDir(FromDir) + FSearchRec.Name, PostFixDir(NewDir) + FSearchRec.Name);
        End;

        FindResult  := FindNext(FSearchRec);
      End;
    End;
{ Now Search the Subdirectories }
    FindResult := FindFirst(PostFixDir(FromDir) + '*.*', faDirectory, DSearchRec);

    While (FindResult=0) Do Begin
      If ((DSearchRec.Attr and faDirectory) = faDirectory)  and (pos('.', DSearchRec.Name) = 0) then begin
         NewToDir   := NewDir;
         NewFromDir := PostFixDir(FromDir) + ExtractCurDirName(DSearchRec.Name);
         { Only Recurse the Directory Tree if INCSUBDIR Flag is set to True }
         If IncSubDir Then Begin
           CopyMoveTreeBranch(DirOp, NewFromDir, NewToDir, IncSubDir, IncFiles);
         End;
      End;
      FindResult := FindNext(DSearchRec);
    End;

  Finally
    FindClose(FSearchRec);
    FindClose(DSearchRec);
  End;

End;

{
 This function extracts the current directory from a directory string
   ie 'C:|UTILS\DELPHIP\FM2'  would provide  'FM2' as a return string
}
Function ExtractCurDirName(TreeBranch :String):String;
Var
   StrPos                             :Integer;
Begin
     IF TreeBranch[Length(TreeBranch)] = '\' then begin
        TreeBranch[Length(TreeBranch)] := ' ';
     End;

     StrPos :=  Pos('\', TreeBranch);
     While StrPos <> 0 Do Begin
         TreeBranch := Copy(TreeBranch, StrPos + 1, 255);
         StrPos :=  Pos('\', TreeBranch);
     End;
     Result := TreeBranch;
End;


Procedure CreateDirList(Var StrList, NameList :TStringList; Const DirPath :String; Const FileAttr :Word);
Var
   FileLine         :String;
   FindResult       :Integer;
   FSearchRec       :TSearchRec;
Begin
  StrList.Clear;
  NameList.Clear;
  FindResult := FindFirst(DirPath ,FileAttr, FSearchRec);
  Try
    While (FindResult=0) Do Begin
      FileLine :=   Format('%12s',[FSearchRec.Name]) + ' '
                  + Format('%12d',[FSearchRec.Size]) + ' '
                  + Format('%15d',[FSearchRec.Time]) + ' '
                  + Chr(FSearchRec.Attr);
      StrList.Add(FileLine);
      NameList.Add(Format('%12s',[FSearchRec.Name]));
{      NameList.Add(FSearchRec.Name);}
      FindResult := FindNext(FSearchRec);
    End;
  Finally
  End;
End;

Procedure CheckforInterrupt(Key :Char; ObjectToInterrupt :String);
Begin
    If Key = Chr(VK_ESCAPE) then begin
      If ObjectToInterrupt = 'FindFile' Then Begin
        FindFileDlg.FileScan1.Interrupted := True;
      End;
      Interrupt_Flag := True;
    End;
End;


function WinExecAndWait(Const CmdStr: String; Const ShowMode: Word):Word;
Var
  InstanceId  :THandle;
  CmdLen      :Integer;
  CmdStrPChar :PChar;
Begin
  { Convert CmdStr to a PChar }
  Getmem(CmdStrPChar,sizeof(CmdStr));
  StrPCopy(CmdStrPChar, CmdStr);
  { Try to run the application }
  InstanceId := WinExec(CmdStrPChar, ShowMode);
  { Check to see if it failed (ie < 32) }
  If InstanceId < 32 then Begin
    Result := InstanceID
  End Else Begin
    { If it did not fail then - loop until it is finished (or you app terminates) }
{   Repeat
      Application.ProcessMessages;
    Until Application.Terminated or (GetModuleUsage(InstanceId) = 0);   }
    Result := 32;
  End;
End;



function TADirectoryExists(Name: string): Boolean;
var
  SR: TSearchRec;
begin
  if Name[Length(Name)] = '\' then Dec(Name[0]);
  if (Length(Name) = 2) and (Name[2] = ':') then begin
    Result := True;  { I assume that any drive is a directory }
  End else Begin
    Result := FindFirst(Name, faDirectory, SR) = 0;
{ If Length is not greater then 6 and result is true - then it is the root of
    a particular drive - so bother to check it - because there is a bug with
    the root directories
}
    If Result then begin
      Result := Result and (SR.Attr and faDirectory <> 0);
    End;
  End;
end;



end.
